@@ -26,10 +26,9 @@ from copy import copy, deepcopy
 
 from . import compiler
 from .colorizer import ColorScheme, setcolor
-from .coreutils import ismacroimport
+from .coreutils import ismacroimport, isfutureimport, inject_after_futureimports
 from .expander import destructure_candidate, namemacro, parametricmacro
 from .unparser import unparse_with_fallbacks
-from .utils import getdocstring
 
 # --------------------------------------------------------------------------------
 # Private utilities.
@@ -71,10 +70,6 @@ def iswithphase(stmt, *, filename):
 
     return n
 
-def isfutureimport(tree):
-    """Return whether `tree` is a `from __future__ import ...`."""
-    return isinstance(tree, ast.ImportFrom) and tree.module == "__future__"
-
 def extract_phase(tree, *, filename, phase=0):
     """Split `tree` into given `phase` and remaining parts.
 
@@ -142,35 +137,6 @@ def extract_phase(tree, *, filename, phase=0):
     newmodule.body = thisphase
     return newmodule
 
-def split_futureimports(body):
-    """Split `body` into `__future__` imports and everything else.
-
-    `body`: list of `ast.stmt`, the suite representing a module top level.
-
-    Returns `[future_imports, the_rest]`.
-    """
-    k = -1  # ensure `k` gets defined even if `body` is empty
-    for k, bstmt in enumerate(body):
-        if not isfutureimport(bstmt):
-            break
-    if k >= 0:
-        return body[:k], body[k:]
-    return [], body
-
-def inject_after_futureimports(stmt, body):
-    """Inject a statement into `body` after `__future__` imports.
-
-    `body`: list of `ast.stmt`, the suite representing a module top level.
-    `stmt`: `ast.stmt`, the statement to inject.
-    """
-    if getdocstring(body):
-        docstring, *body = body
-        futureimports, body = split_futureimports(body)
-        return [docstring] + futureimports + [stmt] + body
-    else:  # no docstring
-        futureimports, body = split_futureimports(body)
-        return futureimports + [stmt] + body
-
 # --------------------------------------------------------------------------------
 # Public utilities.
 
