@@ -6,23 +6,25 @@
 
 import argparse
 import atexit
-from importlib import import_module
-from importlib.util import resolve_name, module_from_spec
 import os
 import pathlib
 import sys
+from importlib import import_module
+from importlib.util import module_from_spec, resolve_name
 
-from ..coreutils import relativize
-from ..core import MacroApplicationError
-
-from .. import activate  # noqa: F401
 from .. import __version__ as __mcpyrate_version__
+from .. import activate  # noqa: F401
+from ..core import MacroApplicationError
+from ..coreutils import relativize
+from ..pycachecleaner import deletepycachedirs, getpycachedirs
 
-__version__ = "3.0.0"
+__version__ = "3.1.0"
 
 _config_dir = "~/.config/mcpyrate"
 _macropython_module = None  # sys.modules doesn't always seem to keep it, so stash it locally too.
 
+# --------------------------------------------------------------------------------
+
 def import_module_as_main(name, script_mode):
     """Import a module, pretending it's __main__.
 
@@ -153,16 +155,37 @@ def main():
                         help='For use together with "-i". Automatically "import numpy as np", '
                              '"import matplotlib.pyplot as plt", and enable mpl\'s interactive '
                              'mode (somewhat like IPython\'s pylab mode).')
+    parser.add_argument('-c', '--clean', dest='path_to_clean', default=None, type=str, metavar='dir',
+                        help='Delete Python bytecode (`.pyc`) caches inside given directory, recursively, '
+                             'and then exit. This removes the `__pycache__` directories, too. '
+                             'The purpose is to facilitate testing `mcpyrate` and programs that use it, '
+                             'because the expander only runs when the `.pyc` cache for the module being '
+                             'imported is out of date or does not exist. For any given module, once the '
+                             'expander is done, Python\'s import system automatically writes the `.pyc` '
+                             'cache for that module.')
+    parser.add_argument("-n", "--dry-run", dest="dry_run", action="store_true", default=False,
+                        help='For use together with "-c". Just scan for and print the .pyc cache '
+                             'directory paths, don\'t actually clean them.')
     opts = parser.parse_args()
 
+    if opts.path_to_clean:
+        # If an error occurs during cleaning, we just let it produce a standard stack trace.
+        if not opts.dry_run:
+            deletepycachedirs(opts.path_to_clean)
+        else:
+            for x in getpycachedirs(opts.path_to_clean):
+                print(x)
+        sys.exit(0)  # only reached if cleaning (or dry run) successful
+
     if opts.interactive:
-        from .console import MacroConsole
         import readline  # noqa: F401, side effect: enable GNU readline in input()
         import rlcompleter  # noqa: F401, side effects: readline tab completion
+
+        from .console import MacroConsole
         repl_locals = {}
         if opts.pylab:  # like IPython's pylab mode, but we keep things in separate namespaces.
-            import numpy
             import matplotlib.pyplot
+            import numpy
             repl_locals["np"] = numpy
             repl_locals["plt"] = matplotlib.pyplot
             matplotlib.pyplot.ion()
